// File    : TFMTNUM.CPP
// Author  : Eric Woodruff,  CIS ID: 72134,1150
// Updated : Tue 01/24/95 20:37:28
// Note    : Hereby declared public domain
// Compiler: Borland C++ 3.1 to 4.xx
//
// This file contains the member functions for the formatted numeric input
// line.  It formats the displayed data with the proper number of decimal
// places and commas.  The input, display, and cursor position are completely
// controlled by this input line.  The validator is used only to verify proper
// range and to transfer the data as the appropriate numeric type.
//

#include <string.h>

#define Uses_TEvent
#define Uses_TInputLine
#define Uses_TKeys
#define Uses_TRect
#include <tv.h>

#include <tfmtnum.h>

// ****************************************************************************

TFormattedNumeric::TFormattedNumeric(TRect &r, int aMaxLen,
  TValidator *val, short numDec) :
    TInputLine(r, aMaxLen, val),
    numDecimals(numDec)
{
}

void TFormattedNumeric::handleEvent(TEvent& event)
{
    char temp[64] = " ", *p;
    int  len;

    if(data[0] == EOS)
        strcpy(data, "0");

    if(state & sfSelected)
        switch(event.what)
        {
            case evMouse:
                // Ignore all focused mouse events.  The mouse cannot delete
                // or select any part of the text.
                clearEvent(event);
                break;

            case evKeyboard:
                switch(event.keyDown.charScan.charCode)
                {
                    case '+':
                    case '-':
                        // Toggle the sign.  I made it a toggle so that
                        // you don't have to delete everything to add
                        // a sign that you forgot.
                        if(data[0] != '-' && data[0] != '+')
                            if(!strcmp(data, "0"))
                                data[0] = EOS;
                            else
                                memmove(data + 1, data, strlen(data) + 1);

                        if(data[0] != event.keyDown.charScan.charCode)
                            data[0] = event.keyDown.charScan.charCode;
                        else
                            memmove(data, data + 1, strlen(data));

                        clearEvent(event);
                        break;

                    case '0':
                    case '1':
                    case '2':
                    case '3':
                    case '4':
                    case '5':
                    case '6':
                    case '7':
                    case '8':
                    case '9':
                        temp[0] = event.keyDown.charScan.charCode;
                        if(!strcmp(data, "0"))
                            data[0] = EOS;

                        // A zero can't immediately follow the sign.
                        if(temp[0] != '0' || (temp[0] == '0' &&
                          data[0] != '-' && data[0] != '+'))
                            strcat(data, temp);

                        clearEvent(event);
                        break;

                    case '.':
                        if(numDecimals && !strchr(data, '.'))
                        {
                            temp[0] = '.';
                            strcat(data, temp);
                        }
                        clearEvent(event);
                        break;

                    case 8:     // Backspace
                        len = strlen(data) - 1;
                        data[len] = EOS;
                        if(data[1] == EOS && (data[0] == '-' ||
                          data[0] == '+'))
                            data[0] = EOS;
                        clearEvent(event);
                        break;

                    case 25:    // Ctrl+Y - Delete all (just like TInputLine)
                        data[0] = EOS;
                        clearEvent(event);
                        break;

                    default:
                        // Ignore everything else except focus changes.
                        if(event.keyDown.keyCode != kbEnter &&
                           event.keyDown.keyCode != kbEsc &&
                           event.keyDown.keyCode != kbTab &&
                           event.keyDown.keyCode != kbShiftTab)
                            clearEvent(event);
                        break;
                }

                // If the event was handled, validate the input.  If it is
                // too long or has too many decimal places, drop the added
                // character.
                if(event.what == evNothing)
                {
                    p = strchr(data, '.');

                    // Check the formatted display length, not the
                    // actual string length.  Then check decimal places
                    // if there are any.
                    if(formatData(temp, len) > maxLen || (p &&
                      strlen(p + 1) > numDecimals))
                    {
                        len = strlen(data) - 1;
                        data[len] = EOS;
                    }
                }
                drawView();
                break;
        }

    TInputLine::handleEvent(event);
}

void TFormattedNumeric::draw(void)
{
    TDrawBuffer buf;
    char disp[64], color = getColor(1);
    int  i, decPlaces;

    formatData(disp, decPlaces);

    i = size.x - strlen(disp) - 1;
    buf.moveChar(0, ' ', color, size.x);
    buf.moveStr(i, disp, color);
    writeLine(0, 0, size.x, 1, buf);

    // Set cursor position.  Position is set on the decimal point when
    // entering digits to the left of the decimal place (or the rightmost
    // digit if there are no decimal places).  Position is set on the
    // appropriate digit when entering actual decimal places.
    if(size.x - 2 - numDecimals + decPlaces > size.x - 2)
        setCursor(size.x - 2, 0);
    else
        setCursor(size.x - 2 - numDecimals + decPlaces, 0);
}

int TFormattedNumeric::formatData(char *s, int &dec, Boolean addCommas)
{
    char *p, *p2;
    short i = 0;

    dec = 0;

    if(data[0])
    {
        strcpy(s, data);

        // Skip sign?
        if(*s == '-' || *s == '+')
            s++;
    }
    else
        strcpy(s, "0");

    if(numDecimals)
    {
        p = strchr(s, '.');
        if(!p)
        {
            p = s + strlen(s);      // Add a decimal point.
            *p = '.';
            *(p + 1) = EOS;
        }
        else
            dec = strlen(p);        // Number of decimal places including '.'

        // Format the display string with the proper number of decimal places.
        p2 = p + 1;
        while(*p2)
        {
            p2++;
            i++;
        }

        if(i < numDecimals)
        {
            while(i++ < numDecimals)
                *p2++ = '0';

            *p2 = EOS;
        }
    }
    else
        p = s + strlen(s);      // Add a decimal point.

    // Format the display string with commas if necessary.
    if(addCommas)
        for(i = 3; p != s; i--, p--)
            if(!i)
            {
                memmove(p + 1, p, strlen(p) + 1);
                *p = ',';
                i = 3;
            }

    return strlen(s);
}

// Set state is overridden to maintain proper cursor position when moving
// into the field.  It does this by stripping decimal places if they are all
// zero (i.e. ".00").
void TFormattedNumeric::setState(ushort aState, Boolean enable)
{
    char *p;

    TInputLine::setState(aState, enable);

    if(data[0] && aState == sfFocused && enable)
    {
        p = data + strlen(data) - 1;

        while(p != data && *p == '0')
            p--;

        // If there are no non-zero decimal places, strip them all off so
        // that the cursor gets positioned correctly.
        if(*p == '.')
        {
            *p = EOS;
            drawView();
        }
    }
}

#if !defined(NO_STREAMABLE)

void TFormattedNumeric::write( opstream& os )
{
    TInputLine::write(os);
    os << numDecimals;
}

void *TFormattedNumeric::read( ipstream& is )
{
    TInputLine::read(is);
    is >> numDecimals;
    return this;
}

TStreamable *TFormattedNumeric::build()
{
    return new TFormattedNumeric( streamableInit );
}

#endif
